@@ -94,7 +94,7 @@ static struct mapping_t ComposeHelp[] = {
   { N_("Attach file"),  OP_COMPOSE_ATTACH_FILE },
   { N_("Descrip"), OP_COMPOSE_EDIT_DESCRIPTION },
   { N_("Help"),    OP_HELP },
-  { NULL }
+  { NULL,	0 }
 };
 
 static void snd_entry (char *b, size_t blen, MUTTMENU *menu, int num)
@@ -1073,7 +1073,7 @@ int mutt_compose_menu (HEADER *msg,   /* structure for new message */
 	    FREE (&idx[idxlen]);
 	    continue;
 	  }
-	  fclose (fp);
+	  safe_fclose (&fp);
 
 	  if ((idx[idxlen]->content = mutt_make_file_attach (fname)) == NULL)
 	  {
